
#-----------------------------------------------------------------------------
#  model1  (ver. 0.1) -- Estimating Agglomeration Model 1
#  Copyright (C) 2007 Giulio Bottazzi and Angelo Secchi

#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  (version 2) as published by the Free Software Foundation;
  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#------------------------------------------------------------------------------



import string
from math import *
from scipy import special
from scipy.optimize import fmin_bfgs
from scipy.optimize.minpack import leastsq
from scipy.stats import chi2
from scipy.stats import histogram2
from scipy.special.orthogonal import poch
from numpy import *
import shelve
import Scientific.IO.ArrayIO


#define polya probability function
#---------------------------------


def polya(n,b):
    a=1.0
    probs=exp(((special.gammaln(N+1)+special.gammaln(L*(a/b))+special.gammaln((a/b)+n)+special.gammaln((a/b)*(L-1)+N-n))-(special.gammaln(L*(a/b)+N)+special.gammaln(a/b)+special.gammaln(n+1)+special.gammaln(N-n+1)+special.gammaln(L*(a/b)-(a/b)))))
    return probs


#theoretical frequencies from different Models
#---------------------------------------------

def model(b):
    freq=[]
    freq = [ L*polya(x,b) for x in range(nmax) ]
    classfreq = [ sum(freq[2**k-1:2**(k+1)-1]) for k in range(maxindex) ]
    classfreq.append(1.-sum(classfreq))
    return array(classfreq)


#Optimizing b parameter 
#----------------------------

#Minimize Chis^2 using Levenberg-Marquardt alghoritm
#----------------------------------------------------

def thempdiff(b):
    freq      = [ polya(x,b) for x in range(nmax) ]
    classfreq = [ sum(freq[2**k-1:2**(k+1)-1]) for k in range(maxindex) ]
    if sum(classfreq) != 1.0:
        classfreq=classfreq/sum(classfreq)
    classfreqsq = sqrt(array(classfreq))
    return classfreqsq - empfreq[:-1]/classfreqsq


def min_LM(thempdiff):
    results=leastsq(thempdiff,x0=init) #Levenberg-Marquardt algorithm
    xopt=results[0]
    freq      = [ polya(x,xopt) for x in range(nmax) ]
    classfreq = [ sum(freq[2**k-1:2**(k+1)-1]) for k in range(maxindex) ]
    if sum(classfreq) != 1.0:
        classfreq=classfreq/sum(classfreq)
    fss=sum(((classfreq - empfreq[:-1])**2)/classfreq)
    sav=sum(abs((classfreq - empfreq[:-1])))/maxindex
    pvalue=chi2.pdf(fss,maxindex-1)
    return xopt,pvalue,fss,sav,(classfreq*L)


#Main Loop
#----------

storage=shelve.open("shelve_regression")
matricione=storage["matricione"]
storage.close()

nsec=len(matricione[1])

risultati_LM=[]
mod1=[]
emp={}
t1={}

for sec in range(nsec):
    storage=shelve.open("shelve_regression")
    matricione=storage["matricione"]
    storage.close()
    empdata=matricione[:,sec]
    L=len(matricione)
    N=sum(empdata)
    nmax= int(max(empdata))
    maxindex=int(ceil(log(nmax+2)/log(2)-1))
    empfreq=histogram2(empdata,[ 2**k-1  for k in range(maxindex) ])/float(L)
    empfreq=hstack((empfreq,array(0.0)))
    init=[0.8]
    temp=min_LM(thempdiff)
    #res=transpose(hstack((temp[0],temp[1],temp[3]))) #temp[0]=coefficient, temp[1]=p-value, temp[2]=SAD
    res=transpose(hstack((temp[0],temp[3])))
    risultati_LM.append(res)
    print '\n#######  SECTOR ',sec+1,' #######'
    print '\n Model 1 \n'
    print 'b =',res[0]
    print '\n AAD=', res[1]
    emp[sec]=empfreq[:-1]*L
    t1[sec]=temp[4] 

output=file("results/model1_coef.dat","w")
print >> output, "Coef. b \t AAD"
for i in risultati_LM:
     print >> output, i

output.close()


storage=shelve.open("shelve_results")


storage['emp']=emp
storage['t1']=t1

storage.close()


#Creating Shelve
#----------------

storage=shelve.open("shelve_results")

emp=storage['emp']
t1=storage['t1']

storage.close()


Emp=[]
T1=[]

for sec in range(nsec):
    empirica=emp[sec]
    teorica1=t1[sec]
    Emp.append(empirica)
    T1.append(teorica1)


# Files per i Plots

output=file("results/model1_emp.txt","w")

for sec in Emp:
    print >> output, str(sec)[1:-1]

output.close()

output=file("results/model1_theo.txt","w")

for sec in T1:
    print >> output, str(sec).replace('\n','')[1:-1]

output.close()


